package ru.other.company.loy.service.provider;

import java.math.BigDecimal;
import ru.crystals.pos.api.ext.loyal.dto.Discount;
import ru.crystals.pos.api.ext.loyal.dto.LoyaltyResult;
import ru.crystals.pos.spi.receipt.LineItem;
import ru.crystals.pos.spi.receipt.Receipt;

/**
 * Распределятель по чеку скидок от внешней системы
 */
public class DiscountDistributor {

    private ExampleLoyProvider plugin;

    public DiscountDistributor(ExampleLoyProvider plugin) {
        this.plugin = plugin;
    }
    /**
     * Распределяет скидки по чеку. В настоящем плагине сюда бы передавался ещё и ответ от внешней системы.
     * Примерный же распределятель всегда выдаёт 5% скидки на каждую позицию.
     * @param target результаты расчета скидок плагином лояльности
     * @param receipt чек
     */
    void distribute(LoyaltyResult target, Receipt receipt) {
        for (LineItem position : receipt.getLineItems()) {
            if (position == null || !position.isDiscountable()) {
                // на эту позицию нельзя давать скидок
                continue;
            }

            // оригинальная стоимость позиции
            BigDecimal originalSum = position.getSum().add(position.getDiscountValue());

            // минимально допустимая стоимость позиции
            BigDecimal minSum = plugin.currencyHandler.getPositionSum(position.getMinPrice(), position.getQuantity());

            // максимально допустимый размер скидки, что еще можно дать на эту позицию
            BigDecimal maxDiscount = position.getSum().subtract(minSum);
            if (maxDiscount.compareTo(BigDecimal.ZERO) <= 0) {
                // на эту позицию больше нельзя давать скидок
                continue;
            }

            // скидка в 5% на эту позицию:
            BigDecimal discountValue = plugin.currencyHandler.round(originalSum.multiply(Contstants.DISCOUNT_SHARE));
            if (discountValue.compareTo(maxDiscount) >= 0) {
                // все 5% не можем скинуть из-за ограничений минимальной цены. Отдадим сколько сможем:
                discountValue = maxDiscount;
            }

            // Эта скидка не зависит от карты, поэтому её не проставим здесь
            Discount discount = new Discount(position.getNumber(), null, discountValue, Contstants.REGULAR_DISCOUNT_ID);
            target.getDiscounts().add(discount);
        }
    }
}
