package ru.crystals.test.plugins;

import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.mockito.internal.util.reflection.Whitebox;
import org.slf4j.helpers.NOPLogger;
import ru.crystals.pos.api.ext.loyal.dto.Discount;
import ru.crystals.pos.api.ext.loyal.dto.DiscountCancellationResult;
import ru.crystals.pos.api.ext.loyal.dto.LoyaltyResult;
import ru.crystals.pos.api.ext.loyal.dto.PostDiscountResult;
import ru.crystals.pos.api.ext.loyal.dto.Slip;
import ru.crystals.pos.api.ext.loyal.dto.SlipParagraphType;
import ru.crystals.pos.api.ext.loyal.dto.auxiliaries.FiscalizationFeedback;
import ru.crystals.pos.api.ext.loyal.dto.auxiliaries.PreFiscalizationFeedback;
import ru.crystals.pos.spi.IntegrationProperties;
import ru.crystals.pos.spi.PropertiesReader;
import ru.crystals.pos.spi.receipt.Receipt;

import java.math.BigDecimal;
import java.util.Iterator;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

public class TestLoyaltyPluginTest extends BaseTest {

    private TestLoyaltyPlugin plugin;

    @Before
    public void beforeTest() {
        IntegrationProperties ips = mockIntegrationProperties();
        plugin = new TestLoyaltyPlugin();
        Whitebox.setInternalState(plugin, "integrationProperties", ips);
        Whitebox.setInternalState(plugin, "logger", NOPLogger.NOP_LOGGER);
        plugin.init();
    }

    @Test
    public void testDoDiscount() {
        LoyaltyResult result = plugin.doDiscount(Mockito.mock(Receipt.class));
        assertNotNull(result);
        assertEquals(2, result.getDiscounts().size());
        Iterator<Discount> it = result.getDiscounts().iterator();
        Discount d = it.next();
        assertEquals(1, d.getPosNo());
        assertEquals(BigDecimal.valueOf(4.25), d.getValue());
        assertEquals("test-discount", d.getDiscountId());
        assertEquals("433", d.getCardNumber());

        d = it.next();
        assertEquals(2, d.getPosNo());
        assertEquals(BigDecimal.valueOf(1.23), d.getValue());
        assertEquals("test-discount2", d.getDiscountId());
        assertEquals("483", d.getCardNumber());

        assertEquals(BigDecimal.valueOf(1510, 2), result.getBonusWriteOffsLimits().get("433"));
        assertNotNull(result.getBonusAccrualResults().get("213"));
        assertEquals(BigDecimal.valueOf(18), result.getBonusAccrualResults().get("213").getAmount());
        assertEquals("eeee", result.getBonusAccrualResults().get("213").getTransactionId());
        assertEquals(BigDecimal.valueOf(0.23), result.getBonusesTakenIntoAccount().get("888"));
        assertEquals("attribute-value", result.getExtendedReceiptAttributesMap().get("kkey"));
        assertEquals(1, result.getMessagesToCashier().size());
        assertEquals("Message to cashier", result.getMessagesToCashier().iterator().next());
        assertEquals(1, result.getPrintDocuments().size());
        checkSlip(result.getPrintDocuments().get(0));
    }

    @Test
    public void testOnDiscountCalculationFinished() {
        PostDiscountResult result = plugin.onDiscountCalculationFinished(Mockito.mock(Receipt.class));
        assertNotNull(result);
        assertEquals(BigDecimal.valueOf(49.42), result.getBonusWriteOffLimits().get("433"));
    }

    @Test
    public void testCancelDiscount() {
        DiscountCancellationResult result = plugin.cancelDiscount(Mockito.mock(Receipt.class), Mockito.mock(LoyaltyResult.class));
        assertNotNull(result);
        assertEquals("attr", result.getExtendedReceiptAttributesMap().get("kkk"));
    }

    @Test
    public void testEventBeforeReceiptFiscalized() {
        PreFiscalizationFeedback feedback = plugin.eventBeforeReceiptFiscalized(Mockito.mock(Receipt.class), Mockito.mock(LoyaltyResult.class));
        assertNotNull(feedback);
        assertEquals(1, feedback.getSlips().size());
        checkSlip(feedback.getSlips().get(0));
        assertNotNull(feedback.getBonusAccrualResults().get("433"));
        assertEquals(BigDecimal.valueOf(72.86), feedback.getBonusAccrualResults().get("433").getAmount());
        assertEquals("000", feedback.getBonusAccrualResults().get("433").getTransactionId());
    }

    @Test
    public void testOnReceiptFiscalized() {
        FiscalizationFeedback feedback = plugin.onReceiptFiscalized(Mockito.mock(Receipt.class), Mockito.mock(LoyaltyResult.class));
        assertNotNull(feedback);
        assertEquals(1, feedback.getSlips().size());
        checkSlip(feedback.getSlips().get(0));
        assertNotNull(feedback.getBonusAccrualResults().get("433"));
        assertEquals(BigDecimal.valueOf(72.86), feedback.getBonusAccrualResults().get("433").getAmount());
        assertEquals("000", feedback.getBonusAccrualResults().get("433").getTransactionId());
    }

    private void checkSlip(Slip slip) {
        assertTrue(slip.isDisableCut());
        assertTrue(slip.isDisableLogo());
        assertTrue(slip.isDisableRequisites());
        assertFalse(slip.isSeparated());
        assertEquals(12, slip.getIndex().intValue());
        assertEquals(2,slip.getParagraphs().size());
        assertEquals(SlipParagraphType.TEXT, slip.getParagraphs().get(0).getType());
        assertEquals("line01", slip.getParagraphs().get(0).getPayload());
        assertEquals(SlipParagraphType.TEXT, slip.getParagraphs().get(1).getType());
        assertEquals("line02", slip.getParagraphs().get(1).getPayload());
    }
}
