package ru.crystals.pos.api.ext.loyal.dto;

import java.util.HashSet;
import java.util.Set;

/**
 * Описывает рекламную акцию, которая была применена в расчете чека.
 * @since 0.0.46
 */
public class AppliedAdvertisingAction {
    private Long guid;
    private String externalCode;
    private Set<String> labels;
    private boolean payBonusAllowed = true;
    private boolean accrueBonusAllowed = true;

    /**
     * Возвращает идентификатор рекламной акции. Уникален для всякой акции.
     * @return идентификатор рекламной акции.
     */
    public Long getGuid() {
        return guid;
    }

    /**
     * Устанавливает идентификатор рекламной акции.
     * @param guid идентификатор рекламной акции.
     */
    public void setGuid(Long guid) {
        this.guid = guid;
    }

    /**
     * Возвращает внешний код рекламной акции.
     * @return внешний код рекламной акции.
     */
    public String getExternalCode() {
        return externalCode;
    }

    /**
     * Устанавливает внешний код рекламной акции.
     * @param externalCode внешний код рекламной акции.
     */
    public void setExternalCode(String externalCode) {
        this.externalCode = externalCode;
    }

    /**
     * Возвращает множество меток рекламной акции.<br>
     * Метки это произвольные строки, которые могут добавляться к рекламной акции для её идентификации или отнесения в
     * какую-либо группу. Таким образом, разные рекламные акции могут содержать одинаковые метки.<br>
     * Думайте о них как о тегах в Твиттере.
     * @return множество меток рекламной акции или пустое множество, если таких нет.
     */
    public Set<String> getLabels() {
        if(labels == null) {
            labels = new HashSet<>();
        }
        return labels;
    }

    /**
     * Возвращает флаг-признак, запрещающий оплату бонусными баллами товаров, на которые сработала данная РА
     * @return флаг-признак, запрещающий оплату бонусными баллами товаров, на которые сработала данная РА
     */
    public boolean isPayBonusAllowed() {
        return payBonusAllowed;
    }

    /**
     * Устанавливает флаг-признак, запрещающий оплату бонусными баллами товаров, на которые сработала данная РА
     * @param payBonusAllowed устанавливаемое значение
     */
    public void setPayBonusAllowed(boolean payBonusAllowed) {
        this.payBonusAllowed = payBonusAllowed;
    }

    /**
     * Возвращает флаг-признак, запрещающий начисление бонусных баллов на товары, на которые сработала данная РА
     * @return флаг-признак, запрещающий начисление бонусных баллов на товары, на которые сработала данная РА
     */
    public boolean isAccrueBonusAllowed() {
        return accrueBonusAllowed;
    }

    /**
     * Устанавливает флаг-признак, запрещающий начисление бонусных баллов на товары, на которые сработала данная РА
     * @param accrueBonusAllowed устанавливаемое значение
     */
    public void setAccrueBonusAllowed(boolean accrueBonusAllowed) {
        this.accrueBonusAllowed = accrueBonusAllowed;
    }
}
