package ru.crystals.pos.spi.plugin.card;

import java.util.List;

import ru.crystals.pos.spi.receipt.LineItem;
import ru.crystals.pos.spi.receipt.Receipt;

/**
 * Параметры запроса поиска карты  
 */
public interface CardSearchRequest {
    /**
     * Строка поиска
     * @return строка
     */
    String getSearchString();
    
    /**
     * Источник строки поиска
     * @return {@linkplain CardSearchEventSource} 
     */
    CardSearchEventSource getEventSource(); 

    /**
     * Дополнительные строки поиска.<br>
     * <p>
     * Для источника {@link CardSearchEventSource#MAGNETIC_STRIPE} в этой коллекции содержатся все 4 считанные дорожки магнитной полосы карты.
     * Первой дорожке соответствует  {@code getAdditionalData().get(0)} , последней {@code getAdditionalData().get(3)} 
     * Если значение какой либо дорожки считать не удалось, то значение для дорожки будет null.
     * </p>
     * @return данные четырех дорожек магнитной полосы карты, никогда не вернет null
     */
    List<String> getAdditionalData();

    /**
     * Чек, в который добавляется карта
     * @return чек, в который добавляется карта
     * @since 0.0.46
     */
    Receipt getReceipt();

    /**
     * В случае добавления позиционного купона, тут будет та позиция, на которую добавляем этот купон
     * @return позиция, на которую добавляем позиционный купон
     * @since 0.0.54
     */
    default LineItem getLineItem() {
        return null;
    }
}
