package ru.crystals.pos.spi.plugin.payment;

import java.math.BigDecimal;
import ru.crystals.pos.api.plugin.payment.Payment;
import ru.crystals.pos.spi.receipt.Receipt;

/**
 * Интерфейс запроса на выполнение возврата оплаты.<br>
 * Используется при возврате приобретенных товаров.
 */
public interface RefundRequest {

    /**
     * Получить чек продажи, в котором была выполнена оплата.
     * Может вернуть null (например, при произвольном возврате)
     * 
     * @return чек продажи {@link Receipt}
     */
    Receipt getOriginalReceipt();

    /**
     * Данные выполненной оплаты из чека продажи.
     * 
     * @return {@link Payment}
     */
    Payment getOriginalPayment();
    
    /**
     * Формируемый чек возврата.
     * @return чек возврата {@link Receipt}
     */
    Receipt getRefundReceipt();

    /**
     * Сумма к возврату.
     * <p>
     * Если чек возвращается не целиком, а лишь некоторые позиции чека, то сумма
     * к возврату может быть меньше суммы оплаты из чека продажи.
     * 
     * @return сумма к возврату (scale = 2)
     */
    BigDecimal getSumToRefund();

    /**
     * Интерфейс для оповещения кассы о результатах выполнения возврата.
     * 
     * @return {@link PaymentCallback}
     */
    PaymentCallback getPaymentCallback();

}
