package ru.crystals.pos.spi.receipt;

import java.math.BigDecimal;
import ru.crystals.pos.api.ext.loyal.dto.Discount;

/**
 * Описывает применившуюся в ходе расчета чека скидку.
 * @since 0.0.46
 * @see ru.crystals.pos.api.ext.loyal.dto.Discount
 */
public interface AppliedDiscount {
    /**
     * Возвращает величину предоставленной скидки.
     * @return величина предоставленной скидки в рублях.
     */
    BigDecimal getAmount();

    /**
     * Возвращает количество товара в позиции, на которое предоставлена данная скидка.
     * @return количество товара в позиции, на которое предоставлена данная скидка. Если товар весовой, это килограммы, если товар штучный,
     * это единицы товара. Естественно, таком случае значение будет целым.
     */
    BigDecimal getQuantity();

    /**
     * GUID рекламной акции, срабатывание которой привело к образованию данной скидки.
     * @return GUID рекламной акции, срабатывание которой привело к данной скидке.
     */
    Long getAdvertisingActionGuid();

    /**
     * Идентификатор скидки. Может быть не у всех скидок.
     * @return идентификатор скидки или null, если такого нет.
     * @see Discount#getDiscountId()
     */
    String getDiscountId();
}
